package com.sample.wui.jbpm;


import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.Context;
import javax.naming.InitialContext;

import org.drools.runtime.KnowledgeRuntime;
import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.eventmessaging.EventResponseHandler;
import org.jbpm.eventmessaging.Payload;
import org.jbpm.process.workitem.wsht.GenericHTWorkItemHandler;
import org.jbpm.task.Content;
import org.jbpm.task.Status;
import org.jbpm.task.Task;
import org.jbpm.task.TaskService;
import org.jbpm.task.event.TaskEventKey;
import org.jbpm.task.event.entity.TaskCompletedEvent;
import org.jbpm.task.event.entity.TaskEvent;
import org.jbpm.task.event.entity.TaskFailedEvent;
import org.jbpm.task.event.entity.TaskSkippedEvent;
import org.jbpm.task.service.responsehandlers.AbstractBaseResponseHandler;
import org.jbpm.task.utils.ContentMarshallerHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MyHtWorkItemHandler extends GenericHTWorkItemHandler {

	/** The Constant log. */
	private static final Logger log = LoggerFactory
			.getLogger(MyHtWorkItemHandler
					.class);
	
	private static final String UTIL_SERVICE_JNDI = "java:global/seamWUI/seamWUI.jar/jbpmUtilService";
	
	public MyHtWorkItemHandler(TaskService client, KnowledgeRuntime session) {
		super(client, session);
		// TODO Auto-generated constructor stub
	}

	
	public MyHtWorkItemHandler(KnowledgeRuntime session) {
		super(session);
		// TODO Auto-generated constructor stub
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.jbpm.process.workitem.wsht.GenericHTWorkItemHandler#registerTaskEvents
	 * ()
	 */
	@Override
	protected void registerTaskEvents() {
		TaskCompletedHandler eventResponseHandler = new TaskCompletedHandler();
		TaskEventKey key = new TaskEventKey(TaskCompletedEvent.class, -1);
		getClient().registerForEvent(key, false, eventResponseHandler);
		eventHandlers.put(key, eventResponseHandler);
		key = new TaskEventKey(TaskFailedEvent.class, -1);
		getClient().registerForEvent(key, false, eventResponseHandler);
		eventHandlers.put(key, eventResponseHandler);
		key = new TaskEventKey(TaskSkippedEvent.class, -1);
		getClient().registerForEvent(key, false, eventResponseHandler);
		eventHandlers.put(key, eventResponseHandler);
	}

//	private void workaroundTaskServivceProblem(){
//		JbpmUtilService jbpmUtilService = (JbpmUtilService) lookup(UTIL_SERVICE_JNDI);
//		setClient(jbpmUtilService.getLocalTaskService());
//
//	}

	private class TaskCompletedHandler extends AbstractBaseResponseHandler
			implements EventResponseHandler {

		public void execute(Payload payload) {
			TaskEvent event = (TaskEvent) payload.get();
			final long taskId = event.getTaskId();
			log.info("<HANDLER> event received ");
			try {
				log.info("<HANDLER> "+event.getTaskEventType().getDeclaringClass().getSimpleName());
			} catch (Exception e) {
				log.warn("<HANDLER> problem with gettin event type "+e.getClass().getSimpleName()+" "+e.getMessage());
			}
			if (isOwningSessionOnly()
					&& (session instanceof StatefulKnowledgeSession)) {
				if (((StatefulKnowledgeSession) session).getId() != event
						.getSessionId()) {
					return;
				}
			}

			if (isLocal()) {
				handleCompletedTask(taskId);
			} else {
				Runnable runnable = new Runnable() {

					public void run() {
						handleCompletedTask(taskId);
					}
				};
				new Thread(runnable).start();
			}
		}

		public boolean isRemove() {
			return false;
		}
		
		
		public void handleCompletedTask(long taskId) {
//			workaroundTaskServivceProblem();
			Task task = getClient().getTask(taskId);

			long workItemId = task.getTaskData().getWorkItemId();
			log.info("<HANDLER> task.getTaskData().getStatus() = "
					+ task.getTaskData().getStatus());
			Object result = getResult(task);
			
			if (task.getTaskData().getStatus() == Status.Completed) {
				String userId = task.getTaskData().getActualOwner().getId();
				Map<String, Object> results = new HashMap<String, Object>();
				results.put("ActorId", userId);
				if(result !=null){
					results.put("Result", result);
					if (result instanceof Map) {
						Map<?, ?> map = (Map<?, ?>) result;
						for (Map.Entry<?, ?> entry : map.entrySet()) {
							if (entry.getKey() instanceof String) {
								results.put((String) entry.getKey(),
										entry.getValue());
							}
						}
					}

					session.getWorkItemManager().completeWorkItem(
							task.getTaskData().getWorkItemId(), results);
				} else {
					session.getWorkItemManager().completeWorkItem(workItemId,
							results);
				}
			} else {
				log.info("<HANDLER> abort work item");
				session.getWorkItemManager().abortWorkItem(workItemId);
			}
		}

		private Object getResult(Task task) {
			long contentId = task.getTaskData().getOutputContentId();
			Object result = null;
			if (contentId != -1) {
				Content content = getClient().getContent(contentId);
				result = ContentMarshallerHelper.unmarshall(
						content.getContent(), session.getEnvironment(),
						getClassLoader());
			}
			log.info("<HANDLER> result " +result);
			if(result!=null){
				log.info("<HANDLER> result class: "+result.getClass().getSimpleName());
				if(result instanceof Map){
					@SuppressWarnings("rawtypes")
					Map mapa = (Map) result;
					for(Object key :mapa.keySet()){
						log.info("<HANDLER> <"+key+" , "+mapa.get(key)+">");
					}
				}
			}
			return result;
		}
	}
	
	private Object lookup(String jndiName){

        final Hashtable<String, Object> jndiProperties = new Hashtable<String, Object>();
        jndiProperties.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming");
        
        jndiProperties.put("jboss.naming.client.ejb.context", true);
        //jndiProperties.put(Context.INITIAL_CONTEXT_FACTORY, "org.jboss.naming.remote.client.InitialContextFactory");
        jndiProperties.put(Context.PROVIDER_URL, "remote://localhost:4447");		
		try {
			Context ctx = new InitialContext(jndiProperties);
			return ctx.lookup(jndiName);
		} catch (Exception e) {
			throw new IllegalStateException("Problem with jndi lookup for "+jndiName);
		}
		
	}

}
